/*-
 * <<
 * UAVStack
 * ==
 * Copyright (C) 2016 - 2017 UAVStack
 * ==
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * >>
 */

package com.creditease.uav.datastore.sql;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

public interface TypeHandler {

    /**
     * Sets a parameter on a prepared statement
     * 
     * @param ps
     *            - the prepared statement
     * @param i
     *            - the parameter index
     * @param parameter
     *            - the parameter value
     * @param jdbcType
     *            - the JDBC type of the parameter
     * 
     * @throws SQLException
     *             if setting the parameter fails
     */
    public void setParameter(PreparedStatement ps, int i, Object parameter) throws SQLException;

    /**
     * Gets a column from a result set
     * 
     * @param rs
     *            - the result set
     * @param columnName
     *            - the column name to get
     * 
     * @return - the column value
     * 
     * @throws SQLException
     *             if getting the value fails
     */
    public Object getResult(ResultSet rs, String columnName) throws SQLException;

    /**
     * Gets a column from a result set
     * 
     * @param rs
     *            - the result set
     * @param columnIndex
     *            - the column to get (by index)
     * 
     * @return - the column value
     * 
     * @throws SQLException
     *             if getting the value fails
     */
    public Object getResult(ResultSet rs, int columnIndex) throws SQLException;

    /**
     * Gets a column from a callable statement
     * 
     * @param cs
     *            - the statement
     * @param columnIndex
     *            - the column to get (by index)
     * 
     * @return - the column value
     * 
     * @throws SQLException
     *             if getting the value fails
     */
    public Object getResult(CallableStatement cs, int columnIndex) throws SQLException;
}
